package com.bwie.auth.service.impl;

import com.baomidou.mybatisplus.core.toolkit.Assert;
import com.bwie.auth.service.AuthService;
import com.bwie.common.constant.JwtConstants;
import com.bwie.common.constant.TokenConstants;
import com.bwie.common.domain.User;
import com.bwie.common.domain.request.LoginRequest;
import com.bwie.common.domain.response.Info;
import com.bwie.common.domain.response.JwtResponse;
import com.bwie.common.redis.RedisCache;
import com.bwie.common.remote.user.UserRemoteService;
import com.bwie.common.result.Result;
import com.bwie.common.utils.IdUtils;
import com.bwie.common.utils.JwtUtils;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.concurrent.TimeUnit;

@Service
public class AuthServiceImpl implements AuthService {
    private final UserRemoteService userRemoteService;
    private final RedisCache redisCache;
    private final HttpServletRequest request;

    public AuthServiceImpl(UserRemoteService userRemoteService, RedisCache redisCache, HttpServletRequest request) {
        this.userRemoteService = userRemoteService;
        this.redisCache = redisCache;
        this.request = request;
    }


    @Override
    public JwtResponse login(LoginRequest loginRequest) {
        Result<User> findname = userRemoteService.findname(loginRequest.getUserName());
        Assert.isTrue(findname.isSuccess(),"登陆失败");
        User data = findname.getData();
        Assert.notNull(data,"用户不存在");
        Assert.isTrue(data.getUserPwd().equals(loginRequest.getUserPwd()),"密码不正确");

        HashMap<String, Object> map = new HashMap<>();
        String key = IdUtils.genId();
        map.put(JwtConstants.USER_KEY,key);
        String token = JwtUtils.createToken(map);
        redisCache.setCacheObject(TokenConstants.LOGIN_TOKEN_KEY+key,data,TokenConstants.EXPIRATION, TimeUnit.SECONDS);

        return JwtResponse.builder()
                .expireTime(TokenConstants.EXPIRATION)
                .token(token)
                .build();
    }

    @Override
    public Info info() {
        User cacheObject = redisCache.getCacheObject(TokenConstants.LOGIN_TOKEN_KEY + request.getHeader(JwtConstants.USER_KEY));

        return Info.builder()
                .userPhone(cacheObject.getUserPhone())
                .userId(cacheObject.getUserId())
                .userName(cacheObject.getUserName())
                .userRole(cacheObject.getUserRole())
                .build();
    }
}
