package com.bwie.car.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Assert;
import com.bwie.car.service.CarService;
import com.bwie.common.constant.JwtConstants;
import com.bwie.common.constant.TokenConstants;
import com.bwie.common.domain.Car;
import com.bwie.common.domain.User;
import com.bwie.common.domain.response.CarRes;
import com.bwie.common.domain.response.Info;
import com.bwie.common.redis.RedisCache;
import com.bwie.common.result.Result;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

@RestController
@ResponseBody
public class CarController {
    private final CarService carService;
    private final RedisCache redisCache;
    private final HttpServletRequest request;

    public CarController(CarService carService, RedisCache redisCache, HttpServletRequest request) {
        this.carService = carService;
        this.redisCache = redisCache;
        this.request = request;
    }

    @PostMapping("addcar")
    public Result addcar(@RequestParam Long goodsId){
        LambdaQueryWrapper<Car> carLambdaQueryWrapper = new LambdaQueryWrapper<>();
        carLambdaQueryWrapper.eq(Car::getGoodsId,goodsId);
        carLambdaQueryWrapper.eq(Car::getUserId,info().getUserId());

        Car one = carService.getOne(carLambdaQueryWrapper);
        if(null==one){
            Car build = Car.builder()
                    .goodsId(goodsId)
                    .userId(info().getUserId()).carNum(1)
                    .build();
            carService.save(build);
        } else {
            Car build = Car.builder()
                    .carId(one.getCarId())
                    .goodsId(goodsId)
                    .userId(info().getUserId())
                    .carNum(one.getCarNum()+1)
                    .build();
           carService.updateById(build);
        }
        return Result.success();
    }

    @GetMapping("show")
    public Result show(){
        List<CarRes> show = carService.show();
        return Result.success(show);
    }

    @PostMapping("del")
    public Result del(@RequestParam Long carId){

        carService.del(carId);
        return Result.success();
    }

    public Info info() {
        User cacheObject = redisCache.getCacheObject(TokenConstants.LOGIN_TOKEN_KEY + request.getHeader(JwtConstants.USER_KEY));

        return Info.builder()
                .userPhone(cacheObject.getUserPhone())
                .userId(cacheObject.getUserId())
                .userName(cacheObject.getUserName())
                .userRole(cacheObject.getUserRole())
                .build();
    }
}
