package com.bwie.car.service;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.bwie.car.mapper.CarMapper;
import com.bwie.common.constant.JwtConstants;
import com.bwie.common.constant.TokenConstants;
import com.bwie.common.domain.Car;
import com.bwie.common.domain.User;
import com.bwie.common.domain.response.CarRes;
import com.bwie.common.domain.response.Info;
import com.bwie.common.redis.RedisCache;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.HashSet;
import java.util.List;

@Service
public class CarServiceImpl extends ServiceImpl<CarMapper, Car> implements CarService {
   private final CarMapper carMapper;
    private final RedisCache redisCache;
    private final HttpServletRequest request;

    public CarServiceImpl(CarMapper carMapper, RedisCache redisCache, HttpServletRequest request) {
        this.carMapper = carMapper;
        this.redisCache = redisCache;
        this.request = request;
    }

    @Override
    public List<CarRes> show() {
        if(!redisCache.hasKey("car:")){
            List<CarRes> show = carMapper.show(info().getUserId());
            HashSet<CarRes> carRes = new HashSet<>();
            for (CarRes res : show) {
                carRes.add(res);
            }

            redisCache.setCacheList("car:",show);
        }
        List<CarRes> cacheList = redisCache.getCacheList("car:");


        return cacheList;
    }

    @Override
    public void del(Long carId) {
        redisCache.deleteObject("car:");
        carMapper.del(carId);
    }

    public Info info() {
        User cacheObject = redisCache.getCacheObject(TokenConstants.LOGIN_TOKEN_KEY + request.getHeader(JwtConstants.USER_KEY));

        return Info.builder()
                .userPhone(cacheObject.getUserPhone())
                .userId(cacheObject.getUserId())
                .userName(cacheObject.getUserName())
                .userRole(cacheObject.getUserRole())
                .build();
    }
}
