package com.bwie.goods.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Assert;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.bwie.common.constant.JwtConstants;
import com.bwie.common.constant.TokenConstants;
import com.bwie.common.domain.Goods;
import com.bwie.common.domain.User;
import com.bwie.common.domain.request.CusReq;
import com.bwie.common.domain.request.ManaReq;
import com.bwie.common.domain.request.PayNum;
import com.bwie.common.domain.response.CusRes;
import com.bwie.common.domain.response.GoodsSave;
import com.bwie.common.domain.response.Info;
import com.bwie.common.redis.RedisCache;
import com.bwie.common.remote.es.EsRemoteService;
import com.bwie.common.result.PageResult;
import com.bwie.common.result.Result;
import com.bwie.goods.mapper.GoodsMapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.List;

@Service
public class GoodsServiceImpl extends ServiceImpl<GoodsMapper, Goods> implements GoodsService {
   private final GoodsMapper goodsMapper;
   private final EsRemoteService esRemoteService;
   private final RedisCache redisCache;
   private final HttpServletRequest request;

    public GoodsServiceImpl(GoodsMapper goodsMapper, EsRemoteService esRemoteService, RedisCache redisCache, HttpServletRequest request) {
        this.goodsMapper = goodsMapper;
        this.esRemoteService = esRemoteService;
        this.redisCache = redisCache;
        this.request = request;
    }

    @Override
    public List<Goods> findall() {
        LambdaQueryWrapper<Goods> goodsLambdaQueryWrapper = new LambdaQueryWrapper<>();
        List<Goods> goods = goodsMapper.selectList(goodsLambdaQueryWrapper);
        return goods;
    }

    @Override
    public void initRedis() {

            List<Goods> findall = findall();
            ArrayList<GoodsSave> goodsSaves = new ArrayList<>();
            for (Goods goods : findall) {
                GoodsSave build = GoodsSave.builder().goodsId(goods.getGoodsId())
                        .goodsPrice(goods.getGoodsPrice())
                        .goodsSave(goods.getGoodsSave())
                        .goodsName(goods.getGoodsName())
                        .build();
                goodsSaves.add(build);
            }
            redisCache.setCacheList("goodsSave:",goodsSaves);
    }

    @Override
    public Result showMana(ManaReq manaReq) {
       Assert.isTrue(info().getUserRole()==1,"抱歉,用户不开放管理界面");

        PageHelper.startPage(manaReq.getPageNum(),manaReq.getPageSize());
        List<Goods> goods = goodsMapper.showMana(manaReq);
        PageInfo<Goods> goodsPageInfo = new PageInfo<>(goods);
        return PageResult.toResult(goodsPageInfo.getTotal(),goods);
    }

    @Override
    public void addgoods(Goods gg) {
        goodsMapper.add(gg);
        ArrayList<Goods> goods1 = new ArrayList<>();
        goods1.add(gg);
        esRemoteService.addb(goods1);
    }

    @Override
    public void delgoods(Goods gg) {
        goodsMapper.deleteById(gg);
        esRemoteService.delb(gg.getGoodsId());
    }

    @Override
    public Result<PageResult<CusRes>> showCus(CusReq cusReq) {
        PageHelper.startPage(cusReq.getPageNum(),cusReq.getPageSize());
        List<CusRes> goods = goodsMapper.showCus(cusReq);
        PageInfo<CusRes> cusResPageInfo = new PageInfo<>(goods);
        return PageResult.toResult(cusResPageInfo.getTotal(),goods);
    }

    @Override
    public void updnum(Long goodsId,Integer goodsNum) {

            goodsMapper.updnum(goodsId,goodsNum);
            esRemoteService.del();
            esRemoteService.addb(findall());

    }

    @Override
    public void addnum(Integer goodsNum, Long goodsId) {
        goodsMapper.addnum(goodsId,goodsNum);
        esRemoteService.del();
        esRemoteService.addb(findall());
    }


    public Info info() {
        User cacheObject = redisCache.getCacheObject(TokenConstants.LOGIN_TOKEN_KEY + request.getHeader(JwtConstants.USER_KEY));

        return Info.builder()
                .userPhone(cacheObject.getUserPhone())
                .userId(cacheObject.getUserId())
                .userName(cacheObject.getUserName())
                .userRole(cacheObject.getUserRole())
                .build();
    }

}
