package com.bwie.mq.consumer;

import com.bwie.common.domain.request.MqReq;
import com.bwie.common.redis.RedisCache;
import com.bwie.mq.mapper.MqMapper;
import com.bwie.mq.util.MsgUtil;
import com.rabbitmq.client.Channel;
import lombok.extern.log4j.Log4j2;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.RabbitHandler;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.stereotype.Component;

import java.io.IOException;
import java.util.HashSet;

@Component
@Log4j2
@RabbitListener(queues = "queue")
public class MqConsumer {
    private final RedisCache redisCache;
    private final MqMapper mqMapper;
    private final MsgUtil msgUtil;

    public MqConsumer(RedisCache redisCache, MqMapper mqMapper, MsgUtil msgUtil) {
        this.redisCache = redisCache;
        this.mqMapper = mqMapper;
        this.msgUtil = msgUtil;
    }

    @RabbitHandler
    public void send(MqReq mqReq, Channel channel, Message message) throws IOException {
        log.info("{}信息为:",mqReq,message.getMessageProperties().getDeliveryTag());
        String messageId = message.getMessageProperties().getMessageId();
        long deliveryTag = message.getMessageProperties().getDeliveryTag();
        if(!redisCache.hasKey("id:"+messageId)){
            redisCache.setCacheObject("id:"+messageId,deliveryTag);
        }
        int size = redisCache.getCacheSet(messageId).size();
        try {
            if(0==size){
                HashSet<String> strings = new HashSet<>();
                strings.add(messageId);
                redisCache.setCacheSet(messageId,strings);
               mqMapper.upd(mqReq.getMiddId(),5);
                log.info("已提交退款");
                Thread.sleep(1800);
                mqMapper.upd(mqReq.getMiddId(),6);
                msgUtil.sendMsg(mqReq.getUserPhone(),"1111");
                log.info("客户您好:已经成功退款");
                channel.basicAck(deliveryTag,false);

            }else {
                log.info("消息重复");
                channel.basicReject(deliveryTag,false);
            }
        } catch (InterruptedException e) {
            Long cacheObject = redisCache.getCacheObject("id:" + messageId);
            if((cacheObject+2)==deliveryTag){
                log.info("消息消费不了 不入队了");
                channel.basicNack(deliveryTag,false,false);
            }else {
                log.info("消息消费不了 重新入队了");
                channel.basicNack(deliveryTag,false,true);
            }
        }
    }








}
